//
//  i2c_lcd.c
//
//  I2C interface type Character LCD module Control
//		LCD controler: ST7032i
//		LCD module   : Low voltage I2C LCD module SB1602B / Strowbery Linux Co.,Ltd.
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.06.28 v0.0  First cording
//	2009.07.02 v0.0  Rewrite & 1st release
//	2009.07.05 v0.0  Compile error Bug fix
//	2009.07.06 v0.0  Startup wait 40ms->100ms
//	2009.08.02 v0.1  HI-TECH C -> ImageCraft C Compiler
//

#include <m8c.h>        // part specific constants and macros
#include "i2c_lcd.h"

//
// Initial command & save registers
//
//#define I2C_LCD_INIT_CONT	32			// initial contrast (0~63)
#define I2C_LCD_INIT_CONT	48			// initial contrast (0~63)
static BYTE i2c_lcd_reg_func = 
		I2C_LCD_INST_FUNC
	 |	I2C_LCD_DL_8BIT
//	 |	I2C_LCD_DL_4BIT
	 |	I2C_LCD_N_2LINE
//	 |	I2C_LCD_N_1LINE
//	 |	I2C_LCD_DH_DOUBLE
	 |	I2C_LCD_DH_NORMAL
//	 |	I2C_LCD_IS_EXTEND
//	 |	I2C_LCD_IS_NORMAL
	;
static BYTE i2c_lcd_reg_bias = 
		I2C_LCD_INST_BIAS
//	 |	I2C_LCD_BS_4
	 |	I2C_LCD_BS_5
//	 |	I2C_LCD_OSC_122HZ
//	 |	I2C_LCD_OSC_131HZ 
//	 |	I2C_LCD_OSC_144HZ
//	 |	I2C_LCD_OSC_161HZ
	 |	I2C_LCD_OSC_183HZ
//	 |	I2C_LCD_OSC_221HZ
//	 |	I2C_LCD_OSC_274HZ
//	 |	I2C_LCD_OSC_347HZ 
//	 |	I2C_LCD_OSC_120HZ
//	 |	I2C_LCD_OSC_133HZ
//	 |	I2C_LCD_OSC_149HZ
//	 |	I2C_LCD_OSC_167HZ
//	 |	I2C_LCD_OSC_192HZ
//	 |	I2C_LCD_OSC_227HZ
//	 |	I2C_LCD_OSC_277HZ
	;
static BYTE i2c_lcd_reg_cont = 
		I2C_LCD_INST_CONT
	 |	(I2C_LCD_C_LOW & I2C_LCD_INIT_CONT)
	;
static BYTE i2c_lcd_reg_pow = 
		I2C_LCD_INST_POW
	 |	I2C_LCD_I_ON					// icon
//	 |	I2C_LCD_I_OFF 
	 |	I2C_LCD_B_ON					// booster
//	 |	I2C_LCD_B_OFF 
	 |	(I2C_LCD_C_HIGH & (I2C_LCD_INIT_CONT >> 4))
	;
static BYTE i2c_lcd_reg_foll = 
		I2C_LCD_INST_FOLL
	 |	I2C_LCD_F_ON					// switch follwer circuit
//	 |	I2C_LCD_F_OFF 
//	 |	I2C_LCD_RAB_1					// x1.0
//	 |	I2C_LCD_RAB_1_25				// x1.25
//	 |	I2C_LCD_RAB_1_5					// x1.5
	 |	I2C_LCD_RAB_1_8					// x1.8
//	 |	I2C_LCD_RAB_2					// x2.0
//	 |	I2C_LCD_RAB_2_5					// x2.5
//	 |	I2C_LCD_RAB_3					// x3.0
//	 |	I2C_LCD_RAB_3_75				// x3.75
	;
static BYTE i2c_lcd_reg_disp = 
		I2C_LCD_INST_DISP
	 |	I2C_LCD_DSP_ON
	 |	I2C_LCD_DSP_OFF
//	 |	I2C_LCD_CSL_ON
	 |	I2C_LCD_CSL_OFF
//	 |	I2C_LCD_BLK_ON
//	 |	I2C_LCD_BLK_OFF
	;

//
// API
//
void I2C_LCD_Delay50uTimes(BYTE bTimes){
	int i;
	
	for(; bTimes > 0; --bTimes) {
		for(i = 0; i < 7; i++) {			// CPU=12MHz
			;
		}
	}
	return;
}

void I2C_LCD_Delay1mTimes(BYTE bTimes){
	for(; bTimes > 0; --bTimes) {
		I2C_LCD_Delay50uTimes(20);
	}
	return;
}

//void I2C_LCD_Start() {				// DEL 2009.09.02
void I2C_LCD_Start(void) {				// ADD 2009.09.02
	I2C_LCD_START();
}

//void I2C_LCD_Stop() {					// DEL 2009.09.02
void I2C_LCD_Stop(void) {				// ADD 2009.09.02
	I2C_LCD_STOP();
}

void I2C_LCD_SendCmdData(BYTE cmd, char data){
	BYTE rc;
	BYTE cmddata[2];
	
	cmddata[0] = cmd;					// I2C_LCD_CB_xxx
	cmddata[1] = data;					// Data byte
	rc = I2C_LCD_WRITE(I2C_LCD_ADDR, cmddata, sizeof(cmddata), I2Cm_1_CompleteXfer);
	I2C_LCD_Delay50uTimes(1);	// 26.3us~
	return;
}

//void I2C_LCD_Init() {					// DEL 2009.09.02
void I2C_LCD_Init(void) {				// ADD 2009.09.02
//	I2C_LCD_Delay1mTimes(40);	// DEL 2009.07.06 40ms
	I2C_LCD_Delay1mTimes(100);	// ADD 2009.07.06
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_bias);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_cont);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_pow);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_foll);
	I2C_LCD_Delay1mTimes(200);	// 200ms
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_disp);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);

	return;
}

// Override stdio -> usage: putchar(),puts(),printf()
//void  putch(char c){								// DEL 2009.08.02
//	I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, c);		// DEL 2009.08.02
//	return;											// DEL 2009.08.02
//}													// DEL 2009.08.02

void I2C_LCD_PrString(CHAR *s) {					// ADD 2009.08.02
	for(; *s != '\0'; s++) {						// ADD 2009.08.02
		I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, *s);	// ADD 2009.08.02
	}												// ADD 2009.08.02
	return;											// ADD 2009.08.02
}

void I2C_LCD_PrCString(const char *s) {				// ADD 2009.08.02
	for(; *s != '\0'; s++) {						// ADD 2009.08.02
		I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, *s);	// ADD 2009.08.02
	}												// ADD 2009.08.02
	return;											// ADD 2009.08.02
}

void I2C_LCD_PrHexByte(BYTE b) {					// ADD 2009.08.02
	char buf[1];									// ADD 2009.08.02
													// ADD 2009.08.02
	itoa(buf, (b >>  4), 16);						// ADD 2009.08.02
	I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, buf[0]);	// ADD 2009.08.02
	itoa(buf, (b & 0xf), 16);						// ADD 2009.08.02
	I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, buf[0]);	// ADD 2009.08.02
	return;											// ADD 2009.08.02
}

void I2C_LCD_PrHexInt(INT i) {  					// ADD 2009.08.02
	I2C_LCD_PrHexByte(i >> 8);						// ADD 2009.08.02
	I2C_LCD_PrHexByte(i & 0xff);					// ADD 2009.08.02
	return;											// ADD 2009.08.02
}													// ADD 2009.08.02

void  I2C_LCD_Position(BYTE bRow, BYTE bCol){
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, I2C_LCD_INST_DADR | (bRow << 6) | (bCol));
	return;
}

void  I2C_LCD_Contrast(BYTE bCont){
	bCont &= 0x3f;
	i2c_lcd_reg_cont = (i2c_lcd_reg_cont & ~I2C_LCD_C_LOW ) | (I2C_LCD_C_LOW & bCont);
	i2c_lcd_reg_pow  = (i2c_lcd_reg_pow  & ~I2C_LCD_C_HIGH) | (bCont >> 4);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_cont);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_pow);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);
	return;
}

void  I2C_LCD_Icon(BYTE bAddr, BYTE bFlag){
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_EXTEND);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, I2C_LCD_INST_ICON | (bAddr & I2C_LCD_ICON_ADDR));
	I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, bFlag);
	I2C_LCD_SendCmdData(I2C_LCD_CB_INST, i2c_lcd_reg_func | I2C_LCD_IS_NORMAL);
	return;
}
